%% Clear Workspace and Close figures
clear; close all; clc;

%% Intialize Laplace variable
s = zpk('s');

freqs = logspace(-1, 3, 1000);

% Noise of the sensors
% Let's define the noise characteristics of the two sensors by choosing $N_1$ and $N_2$:
% - Sensor 1 characterized by $N_1(s)$ has low noise at low frequency (for instance a geophone)
% - Sensor 2 characterized by $N_2(s)$ has low noise at high frequency (for instance an accelerometer)


omegac = 100*2*pi; G0 = 1e-5; Ginf = 1e-4;
N1 = (Ginf*s/omegac + G0)/(s/omegac + 1)/(1 + s/2/pi/100);

omegac = 1*2*pi; G0 = 1e-3; Ginf = 1e-8;
N2 = ((sqrt(Ginf)*s/omegac + sqrt(G0))/(s/omegac + 1))^2/(1 + s/2/pi/4000)^2;

figure;
hold on;
plot(freqs, abs(squeeze(freqresp(N1, freqs, 'Hz'))), '-', 'DisplayName', '$N_1$');
plot(freqs, abs(squeeze(freqresp(N2, freqs, 'Hz'))), '-', 'DisplayName', '$N_2$');
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Magnitude');
hold off;
xlim([freqs(1), freqs(end)]);
legend('location', 'northeast');

% H-Two Synthesis
% As $\tilde{n}_1$ and $\tilde{n}_2$ are normalized white noise: $\Phi_{\tilde{n}_1}(\omega) = \Phi_{\tilde{n}_2}(\omega) = 1$ and we have:
% \[ \sigma_{\hat{x}} = \sqrt{\int_0^\infty |H_1 N_1|^2(\omega) + |H_2 N_2|^2(\omega) d\omega} = \left\| \begin{matrix} H_1 N_1 \\ H_2 N_2 \end{matrix} \right\|_2 \]
% Thus, the goal is to design $H_1(s)$ and $H_2(s)$ such that $H_1(s) + H_2(s) = 1$ and such that $\left\| \begin{matrix} H_1 N_1 \\ H_2 N_2 \end{matrix} \right\|_2$ is minimized.

% For that, we use the $\mathcal{H}_2$ Synthesis.

% We use the generalized plant architecture shown on figure [[fig:h_infinity_optimal_comp_filters]].

% #+name: fig:h_infinity_optimal_comp_filters
% #+caption: $\mathcal{H}_2$ Synthesis - Generalized plant used for the optimal generation of complementary filters
% [[file:figs-tikz/h_infinity_optimal_comp_filters.png]]

% The transfer function from $[n_1, n_2]$ to $\hat{x}$ is:
% \[ \begin{bmatrix} N_1 H_1 \\ N_2 (1 - H_1) \end{bmatrix} \]
% If we define $H_2 = 1 - H_1$, we obtain:
% \[ \begin{bmatrix} N_1 H_1 \\ N_2 H_2 \end{bmatrix} \]

% Thus, if we minimize the $\mathcal{H}_2$ norm of this transfer function, we minimize the RMS value of $\hat{x}$.

% We define the generalized plant $P$ on matlab as shown on figure [[fig:h_infinity_optimal_comp_filters]].

P = [0   N2  1;
     N1 -N2  0];



% And we do the $\mathcal{H}_2$ synthesis using the =h2syn= command.

[H1, ~, gamma] = h2syn(P, 1, 1);



% Finally, we define $H_2(s) = 1 - H_1(s)$.

H2 = 1 - H1;



% The complementary filters obtained are shown on figure [[fig:htwo_comp_filters]].

% The PSD of the noise of the individual sensor and of the super sensor are shown in Fig. [[fig:psd_sensors_htwo_synthesis]].

% The Cumulative Power Spectrum (CPS) is shown on Fig. [[fig:cps_h2_synthesis]].

% The obtained RMS value of the super sensor is lower than the RMS value of the individual sensors.


figure;
hold on;
plot(freqs, abs(squeeze(freqresp(H1, freqs, 'Hz'))), '-', 'DisplayName', '$H_1$');
plot(freqs, abs(squeeze(freqresp(H2, freqs, 'Hz'))), '-', 'DisplayName', '$H_2$');
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Magnitude');
hold off;
xlim([freqs(1), freqs(end)]);
legend('location', 'northeast');



% #+NAME: fig:htwo_comp_filters
% #+CAPTION: Obtained complementary filters using the $\mathcal{H}_2$ Synthesis ([[./figs/htwo_comp_filters.png][png]], [[./figs/htwo_comp_filters.pdf][pdf]])
% [[file:figs/htwo_comp_filters.png]]


PSD_S1 = abs(squeeze(freqresp(N1, freqs, 'Hz'))).^2;
PSD_S2 = abs(squeeze(freqresp(N2, freqs, 'Hz'))).^2;
PSD_H2 = abs(squeeze(freqresp(N1*H1, freqs, 'Hz'))).^2+abs(squeeze(freqresp(N2*H2, freqs, 'Hz'))).^2;

figure;
hold on;
plot(freqs, PSD_S1, '-',  'DisplayName', '$\Phi_{\hat{x}_1}$');
plot(freqs, PSD_S2, '-',  'DisplayName', '$\Phi_{\hat{x}_2}$');
plot(freqs, PSD_H2, 'k-', 'DisplayName', '$\Phi_{\hat{x}_{\mathcal{H}_2}}$');
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Power Spectral Density');
hold off;
xlim([freqs(1), freqs(end)]);
legend('location', 'northeast');



% #+NAME: fig:psd_sensors_htwo_synthesis
% #+CAPTION: Power Spectral Density of the estimated $\hat{x}$ using the two sensors alone and using the optimally fused signal ([[./figs/psd_sensors_htwo_synthesis.png][png]], [[./figs/psd_sensors_htwo_synthesis.pdf][pdf]])
% [[file:figs/psd_sensors_htwo_synthesis.png]]


CPS_S1 = 1/pi*cumtrapz(2*pi*freqs, PSD_S1);
CPS_S2 = 1/pi*cumtrapz(2*pi*freqs, PSD_S2);
CPS_H2 = 1/pi*cumtrapz(2*pi*freqs, PSD_H2);

figure;
hold on;
plot(freqs, CPS_S1, '-',  'DisplayName', sprintf('$\\sigma_{\\hat{x}_1} = %.1e$', sqrt(CPS_S1(end))));
plot(freqs, CPS_S2, '-',  'DisplayName', sprintf('$\\sigma_{\\hat{x}_2} = %.1e$', sqrt(CPS_S2(end))));
plot(freqs, CPS_H2, 'k-', 'DisplayName', sprintf('$\\sigma_{\\hat{x}_{\\mathcal{H}_2}} = %.1e$', sqrt(CPS_H2(end))));
set(gca, 'YScale', 'log'); set(gca, 'XScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Cumulative Power Spectrum');
hold off;
xlim([2e-1, freqs(end)]);
ylim([1e-10 1e-5]);
legend('location', 'southeast');

% H-Infinity Synthesis - method A
% Another objective that we may have is that the noise of the super sensor $n_{SS}$ is following the minimum of the noise of the two sensors $n_1$ and $n_2$:
% \[ \Gamma_{n_{ss}}(\omega) = \min(\Gamma_{n_1}(\omega),\ \Gamma_{n_2}(\omega)) \]

% In order to obtain that ideal case, we need that the complementary filters be designed such that:
% \begin{align*}
%   & |H_1(j\omega)| = 1 \text{ and } |H_2(j\omega)| = 0 \text{ at frequencies where } \Gamma_{n_1}(\omega) < \Gamma_{n_2}(\omega) \\
%   & |H_1(j\omega)| = 0 \text{ and } |H_2(j\omega)| = 1 \text{ at frequencies where } \Gamma_{n_1}(\omega) > \Gamma_{n_2}(\omega)
% \end{align*}

% Which is indeed impossible in practice.

% We could try to approach that with the $\mathcal{H}_\infty$ synthesis by using high order filters.

% As shown on Fig. [[fig:noise_characteristics_sensors]], the frequency where the two sensors have the same noise level is around 9Hz.
% We will thus choose weighting functions such that the merging frequency is around 9Hz.

% The weighting functions used as well as the obtained complementary filters are shown in Fig. [[fig:weights_comp_filters_Hinfa]].


n = 5; w0 = 2*pi*10; G0 = 1/10; G1 = 10000; Gc = 1/2;
W1a = (((1/w0)*sqrt((1-(G0/Gc)^(2/n))/(1-(Gc/G1)^(2/n)))*s + (G0/Gc)^(1/n))/((1/G1)^(1/n)*(1/w0)*sqrt((1-(G0/Gc)^(2/n))/(1-(Gc/G1)^(2/n)))*s + (1/Gc)^(1/n)))^n;

n = 5; w0 = 2*pi*8; G0 = 1000; G1 = 0.1; Gc = 1/2;
W2a = (((1/w0)*sqrt((1-(G0/Gc)^(2/n))/(1-(Gc/G1)^(2/n)))*s + (G0/Gc)^(1/n))/((1/G1)^(1/n)*(1/w0)*sqrt((1-(G0/Gc)^(2/n))/(1-(Gc/G1)^(2/n)))*s + (1/Gc)^(1/n)))^n;

P = [W1a -W1a;
     0    W2a;
     1    0];



% And we do the $\mathcal{H}_\infty$ synthesis using the =hinfsyn= command.

[H2a, ~, gamma, ~] = hinfsyn(P, 1, 1,'TOLGAM', 0.001, 'METHOD', 'ric', 'DISPLAY', 'on');



% #+RESULTS:
% #+begin_example
% [H2a, ~, gamma, ~] = hinfsyn(P, 1, 1,'TOLGAM', 0.001, 'METHOD', 'ric', 'DISPLAY', 'on');
% Resetting value of Gamma min based on D_11, D_12, D_21 terms

% Test bounds:      0.1000 <  gamma  <=  10500.0000

%   gamma    hamx_eig  xinf_eig  hamy_eig   yinf_eig   nrho_xy   p/f
% 1.050e+04   2.1e+01 -3.0e-07   7.8e+00   -1.3e-15    0.0000    p
% 5.250e+03   2.1e+01 -1.5e-08   7.8e+00   -5.8e-14    0.0000    p
% 2.625e+03   2.1e+01   2.5e-10   7.8e+00   -3.7e-12    0.0000    p
% 1.313e+03   2.1e+01 -3.2e-11   7.8e+00   -7.3e-14    0.0000    p
%   656.344   2.1e+01 -2.2e-10   7.8e+00   -1.1e-15    0.0000    p
%   328.222   2.1e+01 -1.1e-10   7.8e+00   -1.2e-15    0.0000    p
%   164.161   2.1e+01 -2.4e-08   7.8e+00   -8.9e-16    0.0000    p
%    82.130   2.1e+01   2.0e-10   7.8e+00   -9.1e-31    0.0000    p
%    41.115   2.1e+01 -6.8e-09   7.8e+00   -4.1e-13    0.0000    p
%    20.608   2.1e+01   3.3e-10   7.8e+00   -1.4e-12    0.0000    p
%    10.354   2.1e+01 -9.8e-09   7.8e+00   -1.8e-15    0.0000    p
%     5.227   2.1e+01 -4.1e-09   7.8e+00   -2.5e-12    0.0000    p
%     2.663   2.1e+01   2.7e-10   7.8e+00   -4.0e-14    0.0000    p
%     1.382   2.1e+01 -3.2e+05#  7.8e+00   -3.5e-14    0.0000    f
%     2.023   2.1e+01 -5.0e-10   7.8e+00    0.0e+00    0.0000    p
%     1.702   2.1e+01 -2.4e+07#  7.8e+00   -1.6e-13    0.0000    f
%     1.862   2.1e+01 -6.0e+08#  7.8e+00   -1.0e-12    0.0000    f
%     1.942   2.1e+01 -2.8e-09   7.8e+00   -8.1e-14    0.0000    p
%     1.902   2.1e+01 -2.5e-09   7.8e+00   -1.1e-13    0.0000    p
%     1.882   2.1e+01 -9.3e-09   7.8e+00   -2.0e-15    0.0001    p
%     1.872   2.1e+01 -1.3e+09#  7.8e+00   -3.6e-22    0.0000    f
%     1.877   2.1e+01 -2.6e+09#  7.8e+00   -1.2e-13    0.0000    f
%     1.880   2.1e+01 -5.6e+09#  7.8e+00   -1.4e-13    0.0000    f
%     1.881   2.1e+01 -1.2e+10#  7.8e+00   -3.3e-12    0.0000    f
%     1.882   2.1e+01 -3.2e+10#  7.8e+00   -8.5e-14    0.0001    f

%  Gamma value achieved:     1.8824
% #+end_example


H1a = 1 - H2a;

figure;

ax1 = subplot(2,1,1);
hold on;
set(gca,'ColorOrderIndex',1)
plot(freqs, 1./abs(squeeze(freqresp(W1a, freqs, 'Hz'))), '--', 'DisplayName', '$w_1$');
set(gca,'ColorOrderIndex',2)
plot(freqs, 1./abs(squeeze(freqresp(W2a, freqs, 'Hz'))), '--', 'DisplayName', '$w_2$');

set(gca,'ColorOrderIndex',1)
plot(freqs, abs(squeeze(freqresp(H1a, freqs, 'Hz'))), '-', 'DisplayName', '$H_1$');
set(gca,'ColorOrderIndex',2)
plot(freqs, abs(squeeze(freqresp(H2a, freqs, 'Hz'))), '-', 'DisplayName', '$H_2$');

hold off;
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
ylabel('Magnitude');
set(gca, 'XTickLabel',[]);
ylim([5e-4, 20]);
legend('location', 'northeast');

ax2 = subplot(2,1,2);
hold on;
set(gca,'ColorOrderIndex',1)
plot(freqs, 180/pi*phase(squeeze(freqresp(H1a, freqs, 'Hz'))), '-');
set(gca,'ColorOrderIndex',2)
plot(freqs, 180/pi*phase(squeeze(freqresp(H2a, freqs, 'Hz'))), '-');
hold off;
xlabel('Frequency [Hz]'); ylabel('Phase [deg]');
set(gca, 'XScale', 'log');
yticks([-360:90:360]);

linkaxes([ax1,ax2],'x');
xlim([freqs(1), freqs(end)]);
xticks([0.1, 1, 10, 100, 1000]);



% #+NAME: fig:weights_comp_filters_Hinfa
% #+CAPTION: Weights and Complementary Fitlers obtained ([[./figs/weights_comp_filters_Hinfa.png][png]], [[./figs/weights_comp_filters_Hinfa.pdf][pdf]])
% [[file:figs/weights_comp_filters_Hinfa.png]]

% We then compute the Power Spectral Density as well as the Cumulative Power Spectrum.


PSD_Ha = abs(squeeze(freqresp(N1*H1a, freqs, 'Hz'))).^2+abs(squeeze(freqresp(N2*H2a, freqs, 'Hz'))).^2;
CPS_Ha = 1/pi*cumtrapz(2*pi*freqs, PSD_Ha);

% H-Infinity Synthesis - method B
% We have that:
% \[ \Phi_{\hat{x}}(\omega) = \left|H_1(j\omega) N_1(j\omega)\right|^2 + \left|H_2(j\omega) N_2(j\omega)\right|^2 \]

% Then, at frequencies where $|H_1(j\omega)| < |H_2(j\omega)|$ we would like that $|N_1(j\omega)| = 1$ and $|N_2(j\omega)| = 0$ as we discussed before.
% Then $|H_1 N_1|^2 + |H_2 N_2|^2 = |N_1|^2$.

% We know that this is impossible in practice. A more realistic choice is to design $H_2(s)$ such that when $|N_2(j\omega)| > |N_1(j\omega)|$, we have that:
% \[ |H_2 N_2|^2 = \epsilon |H_1 N_1|^2 \]

% Which is equivalent to have (by supposing $|H_1| \approx 1$):
% \[ |H_2| = \sqrt{\epsilon} \frac{|N_1|}{|N_2|} \]

% And we have:
% \begin{align*}
%   \Phi_{\hat{x}} &= \left|H_1 N_1\right|^2 + |H_2 N_2|^2 \\
%                 &= (1 + \epsilon) \left| H_1 N_1 \right|^2 \\
%                 &\approx \left|N_1\right|^2
% \end{align*}

% Similarly, we design $H_1(s)$ such that at frequencies where $|N_1| > |N_2|$:
% \[ |H_1| = \sqrt{\epsilon} \frac{|N_2|}{|N_1|} \]

% For instance, is we take $\epsilon = 1$, then the PSD of $\hat{x}$ is increased by just by a factor $\sqrt{2}$ over the all frequencies from the idea case.

% We use this as the weighting functions for the $\mathcal{H}_\infty$ synthesis of the complementary filters.

% The weighting function and the obtained complementary filters are shown in Fig. [[fig:weights_comp_filters_Hinfb]].


epsilon = 2;

W1b = 1/epsilon*N1/N2;
W2b = 1/epsilon*N2/N1;

W1b = W1b/(1 + s/2/pi/1000); % this is added so that it is proper

P = [W1b -W1b;
     0    W2b;
     1    0];



% And we do the $\mathcal{H}_\infty$ synthesis using the =hinfsyn= command.

[H2b, ~, gamma, ~] = hinfsyn(P, 1, 1,'TOLGAM', 0.001, 'METHOD', 'ric', 'DISPLAY', 'on');



% #+RESULTS:
% #+begin_example
% [H2b, ~, gamma, ~] = hinfsyn(P, 1, 1,'TOLGAM', 0.001, 'METHOD', 'ric', 'DISPLAY', 'on');
% Test bounds:      0.0000 <  gamma  <=     32.8125

%   gamma    hamx_eig  xinf_eig  hamy_eig   yinf_eig   nrho_xy   p/f
%    32.812   1.8e+01   3.4e-10   6.3e+00   -2.9e-13    0.0000    p
%    16.406   1.8e+01   3.4e-10   6.3e+00   -1.2e-15    0.0000    p
%     8.203   1.8e+01   3.3e-10   6.3e+00   -2.6e-13    0.0000    p
%     4.102   1.8e+01   3.3e-10   6.3e+00   -2.1e-13    0.0000    p
%     2.051   1.7e+01   3.4e-10   6.3e+00   -7.2e-16    0.0000    p
%     1.025   1.6e+01 -1.3e+06#  6.3e+00   -8.3e-14    0.0000    f
%     1.538   1.7e+01   3.4e-10   6.3e+00   -2.0e-13    0.0000    p
%     1.282   1.7e+01   3.4e-10   6.3e+00   -7.9e-17    0.0000    p
%     1.154   1.7e+01   3.6e-10   6.3e+00   -1.8e-13    0.0000    p
%     1.089   1.7e+01 -3.4e+06#  6.3e+00   -1.7e-13    0.0000    f
%     1.122   1.7e+01 -1.0e+07#  6.3e+00   -3.2e-13    0.0000    f
%     1.138   1.7e+01 -1.3e+08#  6.3e+00   -1.8e-13    0.0000    f
%     1.146   1.7e+01   3.2e-10   6.3e+00   -3.0e-13    0.0000    p
%     1.142   1.7e+01   5.5e-10   6.3e+00   -2.8e-13    0.0000    p
%     1.140   1.7e+01 -1.5e-10   6.3e+00   -2.3e-13    0.0000    p
%     1.139   1.7e+01 -4.8e+08#  6.3e+00   -6.2e-14    0.0000    f
%     1.139   1.7e+01   1.3e-09   6.3e+00   -8.9e-17    0.0000    p

%  Gamma value achieved:     1.1390
% #+end_example


H1b = 1 - H2b;

figure;

ax1 = subplot(2,1,1);
hold on;
set(gca,'ColorOrderIndex',1)
plot(freqs, 1./abs(squeeze(freqresp(W1b, freqs, 'Hz'))), '--', 'DisplayName', '$w_1$');
set(gca,'ColorOrderIndex',2)
plot(freqs, 1./abs(squeeze(freqresp(W2b, freqs, 'Hz'))), '--', 'DisplayName', '$w_2$');

set(gca,'ColorOrderIndex',1)
plot(freqs, abs(squeeze(freqresp(H1b, freqs, 'Hz'))), '-', 'DisplayName', '$H_1$');
set(gca,'ColorOrderIndex',2)
plot(freqs, abs(squeeze(freqresp(H2b, freqs, 'Hz'))), '-', 'DisplayName', '$H_2$');

hold off;
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
ylabel('Magnitude');
set(gca, 'XTickLabel',[]);
ylim([5e-4, 20]);
legend('location', 'northeast');

ax2 = subplot(2,1,2);
hold on;
set(gca,'ColorOrderIndex',1)
plot(freqs, 180/pi*phase(squeeze(freqresp(H1b, freqs, 'Hz'))), '-');
set(gca,'ColorOrderIndex',2)
plot(freqs, 180/pi*phase(squeeze(freqresp(H2b, freqs, 'Hz'))), '-');
hold off;
xlabel('Frequency [Hz]'); ylabel('Phase [deg]');
set(gca, 'XScale', 'log');
yticks([-360:90:360]);

linkaxes([ax1,ax2],'x');
xlim([freqs(1), freqs(end)]);
xticks([0.1, 1, 10, 100, 1000]);



% #+NAME: fig:weights_comp_filters_Hinfb
% #+CAPTION: Weights and Complementary Fitlers obtained ([[./figs/weights_comp_filters_Hinfb.png][png]], [[./figs/weights_comp_filters_Hinfb.pdf][pdf]])
% [[file:figs/weights_comp_filters_Hinfb.png]]


PSD_Hb = abs(squeeze(freqresp(N1*H1b, freqs, 'Hz'))).^2+abs(squeeze(freqresp(N2*H2b, freqs, 'Hz'))).^2;
CPS_Hb = 1/pi*cumtrapz(2*pi*freqs, PSD_Hb);



% #+name: tab:rms_results
% #+caption: RMS value of the estimation error when using the sensor individually and when using the two sensor merged using the optimal complementary filters
% #+RESULTS:
% |              | rms value |
% |--------------+-----------|
% | Sensor 1     |   1.3e-03 |
% | Sensor 2     |   1.3e-03 |
% | H2 Fusion    |   1.2e-04 |
% | H-Infinity a |   2.4e-04 |
% | H-Infinity b |   1.4e-04 |



figure;
hold on;
plot(freqs, PSD_S1, '-',  'DisplayName', '$\Phi_{\hat{x}_1}$');
plot(freqs, PSD_S2, '-',  'DisplayName', '$\Phi_{\hat{x}_2}$');
plot(freqs, PSD_H2, 'k-', 'DisplayName', '$\Phi_{\hat{x}_{\mathcal{H}_2}}$');
plot(freqs, PSD_Ha, 'k--', 'DisplayName', '$\Phi_{\hat{x}_{\mathcal{H}_\infty},a}$');
plot(freqs, PSD_Hb, 'k-.', 'DisplayName', '$\Phi_{\hat{x}_{\mathcal{H}_\infty},b}$');
set(gca, 'XScale', 'log'); set(gca, 'YScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Power Spectral Density');
hold off;
xlim([freqs(1), freqs(end)]);
legend('location', 'northeast');



% #+NAME: fig:comparison_psd_noise
% #+CAPTION: Comparison of the obtained Power Spectral Density using the three methods ([[./figs/comparison_psd_noise.png][png]], [[./figs/comparison_psd_noise.pdf][pdf]])
% [[file:figs/comparison_psd_noise.png]]


figure;
hold on;
plot(freqs, CPS_S1, '-',  'DisplayName', sprintf('$\\sigma_{\\hat{x}_1} = %.1e$', sqrt(CPS_S1(end))));
plot(freqs, CPS_S2, '-',  'DisplayName', sprintf('$\\sigma_{\\hat{x}_2} = %.1e$', sqrt(CPS_S2(end))));
plot(freqs, CPS_H2, 'k-', 'DisplayName', sprintf('$\\sigma_{\\hat{x}_{\\mathcal{H}_2}} = %.1e$', sqrt(CPS_H2(end))));
plot(freqs, CPS_Ha, 'k--', 'DisplayName', sprintf('$\\sigma_{\\hat{x}_{\\mathcal{H}_\\infty, a}} = %.1e$', sqrt(CPS_Ha(end))));
plot(freqs, CPS_Hb, 'k-.', 'DisplayName', sprintf('$\\sigma_{\\hat{x}_{\\mathcal{H}_\\infty, b}} = %.1e$', sqrt(CPS_Hb(end))));
set(gca, 'YScale', 'log'); set(gca, 'XScale', 'log');
xlabel('Frequency [Hz]'); ylabel('Cumulative Power Spectrum');
hold off;
xlim([2e-1, freqs(end)]);
ylim([1e-10 1e-5]);
legend('location', 'southeast');
